'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = flowTypeHandler;

var _recast = require('recast');

var _recast2 = _interopRequireDefault(_recast);

var _getFlowType = require('../utils/getFlowType');

var _getFlowType2 = _interopRequireDefault(_getFlowType);

var _getPropertyName = require('../utils/getPropertyName');

var _getPropertyName2 = _interopRequireDefault(_getPropertyName);

var _getFlowTypeFromReactComponent = require('../utils/getFlowTypeFromReactComponent');

var _getFlowTypeFromReactComponent2 = _interopRequireDefault(_getFlowTypeFromReactComponent);

var _resolveToValue = require('../utils/resolveToValue');

var _resolveToValue2 = _interopRequireDefault(_resolveToValue);

var _setPropDescription = require('../utils/setPropDescription');

var _setPropDescription2 = _interopRequireDefault(_setPropDescription);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * Copyright (c) 2015, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 *
 */
var types = _recast2.default.types.namedTypes;

function setPropDescriptor(documentation, path) {
  if (types.ObjectTypeSpreadProperty.check(path.node)) {
    var name = path.get('argument').get('id').get('name');
    var resolvedPath = (0, _resolveToValue2.default)(name);

    if (resolvedPath && types.TypeAlias.check(resolvedPath.node)) {
      var right = resolvedPath.get('right');
      (0, _getFlowTypeFromReactComponent.applyToFlowTypeProperties)(right, function (propertyPath) {
        setPropDescriptor(documentation, propertyPath);
      });
    } else {
      documentation.addComposes(name.node.name);
    }
  } else if (types.ObjectTypeProperty.check(path.node)) {
    var type = (0, _getFlowType2.default)(path.get('value'));
    var propDescriptor = documentation.getPropDescriptor((0, _getPropertyName2.default)(path));
    propDescriptor.required = !path.node.optional;
    propDescriptor.flowType = type;

    // We are doing this here instead of in a different handler
    // to not need to duplicate the logic for checking for
    // imported types that are spread in to props.
    (0, _setPropDescription2.default)(documentation, path);
  }
}

/**
 * This handler tries to find flow Type annotated react components and extract
 * its types to the documentation. It also extracts docblock comments which are
 * inlined in the type definition.
 */
function flowTypeHandler(documentation, path) {
  var flowTypesPath = (0, _getFlowTypeFromReactComponent2.default)(path);

  if (!flowTypesPath) {
    return;
  }

  (0, _getFlowTypeFromReactComponent.applyToFlowTypeProperties)(flowTypesPath, function (propertyPath) {
    setPropDescriptor(documentation, propertyPath);
  });
}